from abc import abstractmethod
from pathlib import Path
import re
import pkg_resources


PLUGIN_ENTRYPOINT = "hmd_lib_remote_deps.handlers"


class RemoteDependencyHandler:
    def __init__(self, prefix: str) -> None:
        self.prefix = prefix

    @abstractmethod
    def __call__(self, spec: str) -> Path:
        raise NotImplementedError

    @abstractmethod
    def get_package_name(self, spec: str) -> str:
        raise NotImplementedError

    @abstractmethod
    def get_package_version(self, spec: str) -> str:
        raise NotImplementedError

    @abstractmethod
    def get_package_version(self, spec: str) -> str:
        raise NotImplementedError

    @abstractmethod
    def get_file_spec(self, spec: str) -> str:
        raise NotImplementedError


class RemoteDependencyManager:
    def __init__(self):
        self.handlers = {}

    def load_plugins(self):
        entrypoints = pkg_resources.iter_entry_points(PLUGIN_ENTRYPOINT)

        for entrypoint in entrypoints:
            handler: RemoteDependencyHandler = entrypoint.load()()
            self.handlers[handler.prefix] = handler

    def get_handler(self, spec: str) -> RemoteDependencyHandler:
        prefix_re = "(?P<prefix>^[a-z]{2,5}:)"

        m = re.match(prefix_re, spec)
        if m is None:
            return None
        groups = m.groupdict()

        prefix = groups["prefix"]

        if prefix is None:
            return
        handler = self.handlers.get(prefix)
        assert handler is not None, f"Missing handler for prefix: {prefix}"

        return handler

    def fetch_dependency(self, spec: str):
        handler = self.get_handler(spec)

        # if get_handler returns None then no prefix is found
        # and we can assume its a local file path
        if handler is None:
            return Path(spec)

        return handler(spec)

    def get_package_name(self, spec: str) -> str:
        handler = self.get_handler(spec)

        if handler is None:
            return

        return handler.get_package_name(spec)

    def get_package_version(self, spec: str) -> str:
        handler = self.get_handler(spec)

        if handler is None:
            return

        return handler.get_package_version(spec)

    def get_file_spec(self, spec: str) -> str:
        handler = self.get_handler(spec)

        if handler is None:
            return spec

        return handler.get_file_spec(spec)
